--[[
Name: FlexBar2 Core
Revision: $Rev$
Author: Cncfanatics
Description: Action Bars manager Mod, Core file
Dependencies: AceLibrary, AceOO-2.0, AceOO-2.0, AceEvent-2.0, AceAddon-2.0,AceDB-2.0, AceDebug-2.0, AceHook-2.1, AceConsole-2.0, AceLocale-2.2, Dewdrop-2.0, Tablet-2.0, FuBarPlugin-2.0
]]
FlexBar2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "AceHook-2.1", "AceDebug-2.0"); -- Note: not registerring AceEvent-2.0 here, but in every class, as the event handling methods are in there anyway.
FlexBar2.OO = AceLibrary("AceOO-2.0");
FlexBar2:RegisterDB("FlexBar2_DB");
FlexBar2:RegisterDefaults("profile",
{
	Controller = "Default", -- Use the default controller supplied by flexbar
	Theme = "Default", -- Use the default theme supplied by flexbar
	EventHandler = "Default", -- Use the default event handler supplied by flexbar
	Button = {},
	GenButtons = {}, -- List of classnames to generate by default
});

FlexBar2.Class = {};
FlexBar2.Class.Controller = {};
FlexBar2.Class.Theme = {};
FlexBar2.Class.EventHandler = {};

function FlexBar2:Startup()
	--[[
	Classes are expected to be registered at this point, we will be using the classes that the user told us to (or 'Default' if he didn't tell us what one to use) and create a mixin with htem which will contain everything that a button needs to work.
	Note: There is only one mixin kept by FlexBar2, if you want, as an addon author, to get buttons for your addon (I'm thinking about FlexTotems and such here) that do not respect the user chosen mixin, you'll have to make a mixin yourself. (Shouldn't be difficult, all registered classes are kept in the FlexBar2.Class table)
	--]]	
	self.Mixin = self.OO.Class(
							FlexBar2.ButtonInterface, -- Inherit from the interface to check validity
							FlexBar2.Class.Controller[self.db.profile.Controller], -- Inherit from the controller (reacts on user input and events)
							FlexBar2.Class.Theme[self.db.profile.Theme], -- Inherit from the theme (Decides the look of the buttons)
							FlexBar2.Class.EventHandler[self.db.profile.EventHandler], -- Inherit from the event handler (Catches (hardware) events (on the button) and passes them to the controller (this being mainly dragging, clicking, etc. as those are not handled by AceEvent))
							"AceEvent-2.0" -- Inherit AceEvent to do a part of the EventHandler's job (yes, i'm lazy)
						);
	self.Buttons = {};
	for i in pairs(self.db.profile.GenButtons) do
		self:Generate(i);
	end
end

function FlexBar2:OnInitialize()
	self:SetDebugging(false);
	self:Startup();
end

function FlexBar2:OnEnable()
end

function FlexBar2:OnDisable()
end

function FlexBar2:OnProfileEnable()
	self:Startup();
end

function FlexBar2:Generate(i, save)
	self:Debug("Generating " .. i);
	self.Buttons[i] = self.Mixin:new();
	self.Buttons[i]:Load(i);
	if(save) then
		self.db.profile.GenButtons[i] = true;
	end
end

function FlexBar2:SetDefaults(Button)
	local X, Y, Attributes;
	if(not self.Buttons[Button]) then
		self:Generate(Button, true);
	end
	if(type(Button) == "number") then
		X = 100 + (38 * (Button - ((math.ceil(Button / 12) - 1) * 12)));
		Y = 100 + (38 * (math.ceil(Button / 12))); -- So anoying that Y has to be negative, but nothing I can do about it as long as I anchor to TOPLEFT
		if(Button >= 1 and Button <= 120) then -- Only set the attributes for a button to act like an actionbutton if the button's number would be a valid action id
			Attributes =
			{
				action = Button,
				type = "action",
			}
		end
	else
		X = 100;
		Y = 100;
	end
	self.db.profile.Button[Button] = 
	{
		Create = true,
		Show = true,
		Lock = false,
		LockIcon = false,
		X = X,
		Y = Y,
		Attributes = Attributes,
	};
	self.Buttons[Button]:LoadSettings();
end



function FlexBar2:RegisterClass(reference)
	--[[
		Registers a class with FlexBar2, the class will be verified for validity and if a class with the same name was not registered before.
		After a class has been registered it will be a valid choice as new handler in the options.
		If the class you are trying to register has the same name as an already existing class FlexBar2 will spit out a nice little error and ignore the class you tried to register.
	--]]
	if(self:xor(reference.Controller,  FlexBar2:xor(reference.Theme,reference.EventHandler))) then
		local ClassType;
		local ClassName;
		if(type(reference.Controller) == "string") then
			ClassType = "Controller";
		elseif(type(reference.EventHandler) == "string") then
			ClassType = "EventHandler";
		elseif(type(reference.Theme) == "string") then
			ClassType = "Theme";
		else
			self:Print("Invalid class reference passed");
			return false;
		end
		if(self.Class[ClassType][reference[ClassType]] == nil) then
			self.Class[ClassType][reference[ClassType]] = reference;
		else
			self:Print("A class with the same name (' " .. reference[ClassType] .. "') has already been registered.");
			return false;
		end
	else
		self:Print("Invalid class reference passed");
		return false;
	end
end

-- Utility functions used on many parts of the addon
function FlexBar2:copyTable(t)
	local x = {}
	for k,v in pairs(t) do
		if(type(v) == "table") then
			x[k] = copyTable(v);
		else
			x[k] = v;
		end
		
	end
	setmetatable(x, getmetatable(t))
	return x
end

function FlexBar2:xor(a, b)
	return not (a ~= nil and a ~= false) == (b ~= nil and b ~= false);
end
