FlexBar2.SettingCache = { Setup = "",};

local function ButtonsAreNotGenerated()
	if(FlexBar2.SettingCache.Setup == "Button") then
		for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
			if(type(FlexBar2.Buttons[v]) ~= "table") then
				return true;
			end
		end
		return false;
	else
		return true;
	end
end

local function ButtonsAreNotCreated()
	if(FlexBar2.SettingCache.Setup == "Button") then
		for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
			if(type(FlexBar2.Buttons[v]) ~= "table" or type(FlexBar2.Buttons[v].Frame) ~= "table") then
				return true;
			end
		end
		return false;
	else
		return true;
	end
end
FlexBar2.OnMenuRequest = 
{
	type = "group",
	name = "FlexBar2",
	desc = "FlexBar2 menu",
	args = 
	{
		{
			type = "group",
			name = "Set up Button(s)",
			desc = "Setup a button or a range of buttons",
			disabled = InCombatLockdown,
			args =
			{
				{
					type = "text",
					name = "Choose button(s)",
					desc = "Choose the buttons to configure",
					order = 1,
					usage = "<Button>",
					validate = function(StrButtonList)
						return FlexBar2:ButtonParser(StrButtonList, true)
					end,
					get = function()
						if(FlexBar2.SettingCache.Setup == "Button") then
							return FlexBar2.SettingCache.StrButtonList;
						end
					end,
					set = function(StrButtonList)
						FlexBar2.SettingCache.Setup = "Button";
						FlexBar2.SettingCache.StrButtonList = StrButtonList;
						FlexBar2.SettingCache.TblButtonList = FlexBar2:ButtonParser(StrButtonList);
					end,
				},
				{
					type = "toggle",
					name = "Generate",
					desc = "Toggle class generation of the selected buttons",
					order = 2,
					disabled = function()
						if(FlexBar2.SettingCache.Setup == "Button") then
							return false;
						else
							return true;
						end
					end,
					get = function()
						if(FlexBar2.SettingCache.Setup == "Button" and FlexBar2.db.profile.GenButtons[FlexBar2.SettingCache.TblButtonList[1]] == true) then
							return true;
						else
							return false;
						end
					end,
					set = function(NewValue)
						for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
							FlexBar2.db.profile.GenButtons[v] = NewValue;
							if(NewValue == true) then
								FlexBar2:Generate(v);
							else
								FlexBar2.db.profile.GenButtons[v] = nil;
								FlexBar2.Buttons[v] = nil;
							end
						end
					end,
				},
				{
					type = "toggle",
					name = "Create",
					desc = "Toggle creation of the selected buttons",
					order = 3,
					disabled = ButtonsAreNotGenerated,
					get = function()
						if(not ButtonsAreNotGenerated()) then
							return FlexBar2.db.profile.Button[FlexBar2.SettingCache.TblButtonList[1]].Create;
						end
					end,
					set = function(NewValue)
						for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
							if(NewValue == true) then
								FlexBar2.Buttons[v]:Create(true);
							else
								FlexBar2.Buttons[v]:Destroy(true);
							end
						end
					end,
				},
				{
					type = "toggle",
					name = "Show",
					desc = "Toggle the visibility of the selected buttons",
					order = 4,
					disabled = ButtonsAreNotCreated,
					get = function()
						if(not ButtonsAreNotCreated()) then
							return FlexBar2.Buttons[FlexBar2.SettingCache.TblButtonList[1]].Settings.Show;
						end
					end,
					set = function(NewValue)
						for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
							 if(NewValue == true) then
								 FlexBar2.Buttons[v]:Show(true);
							 else
								 FlexBar2.Buttons[v]:Hide(true);
							 end
						end
					end,
				},
				{
					type = "toggle",
					name = "Lock",
					desc = "Toggle the locking of the selected buttons",
					order = 5,
					disabled = ButtonsAreNotCreated,
					get = function()
						if(not ButtonsAreNotCreated()) then
							return FlexBar2.Buttons[FlexBar2.SettingCache.TblButtonList[1]].Settings.Lock;
						end
					end,
					set = function(NewValue)
						for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
							 if(NewValue == true) then
								 FlexBar2.Buttons[v]:Lock(true);
							 else
								 FlexBar2.Buttons[v]:Unlock(true);
							 end
						end
					end,
				},
				{
					type = "toggle",
					name = "Icon lock",
					desc = "Toggle the icon lock of the selected buttons",
					disabled = true, -- Not coded yet
					order = 6,
					get = function()
						if(not ButtonsAreNotCreated()) then
							return FlexBar2.Buttons[FlexBar2.SettingCache.TblButtonList[1]].Settings.LockIcon;
						end
					end,
					set = function(NewValue)
						for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
							 if(NewValue == true) then
								 FlexBar2.Buttons[v]:LockIcon(true);
							 else
								 FlexBar2.Buttons[v]:UnlockIcon(true);
							 end
						end
					end,
				},
				{
					type = "group",
					name = "Position",
					desc = "Set the position of the selected buttons on the screen",
					order = 7,
					disabled =  ButtonsAreNotCreated,
					args =
					{
						{
							type = "text",
							name = "X Coordinate",
							desc = "Set the X coordinate(s) of the selected buttons",
							usage = "<Position>",
							disabled = function()
								return FlexBar2.SettingCache.UsingGeneratedPositioning;
							end,
							order = 1,
							get = function()
								if(type(FlexBar2.SettingCache.X) == "table" and FlexBar2.SettingCache.X[FlexBar2.SettingCache.TblButtonList[1]]) then
									return FlexBar2.SettingCache.X[FlexBar2.SettingCache.TblButtonList[1]];
								else
									return FlexBar2.Buttons[FlexBar2.SettingCache.TblButtonList[1]].Settings.X;
								end
							end,
							set = function(Value)
								FlexBar2.SettingCache.X = {};
								for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
									FlexBar2.SettingCache.X[v] = tonumber(Value);
								end
							end,
						},
						{
							type = "text",
							name = "Y Coordinate",
							desc = "Set the Y coordinate(s) of the selected buttons",
							usage = "<Position>",
							disabled = function()
								return FlexBar2.SettingCache.UsingGeneratedPositioning;
							end,
							order = 2,
							get = function()
								if(type(FlexBar2.SettingCache.Y) == "table" and FlexBar2.SettingCache.Y[FlexBar2.SettingCache.TblButtonList[1]]) then
									return FlexBar2.SettingCache.Y[FlexBar2.SettingCache.TblButtonList[1]];
								else
									return FlexBar2.Buttons[FlexBar2.SettingCache.TblButtonList[1]].Settings.Y;
								end
							end,
							set = function(Value)
								FlexBar2.SettingCache.Y = {};
								for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
									FlexBar2.SettingCache.Y[v] = tonumber(Value);
								end
							end,
						},
						{
							type = "toggle",
							name = "Use default position",
							desc = "Set the default positioning options for the selected button(s)",
							disabled = function()
								for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
									if(type(v) ~= "number") then
										return true;
									end
								end
								return false;
							end,
							order = 3,
							get = function()
								return FlexBar2.SettingCache.UsingGeneratedPositioning;
							end,
							set = function(NewValue)
								FlexBar2.SettingCache.UsingGeneratedPositioning = NewValue;
								if(NewValue == true) then
									FlexBar2.SettingCache.X = {};
									FlexBar2.SettingCache.Y = {};
									for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
										FlexBar2.SettingCache.X[v] = 100 + (38 * (v - ((math.ceil(v / 12) - 1) * 12)));
										FlexBar2.SettingCache.Y[v] = 100 + (38 * (math.ceil(v / 12)));
									end
								else
									FlexBar2.SettingCache.X = nil;
									FlexBar2.SettingCache.Y = nil;
								end
								
							end,
						},
						{
							type = "execute",
							name = "Apply",
							desc = "Apply the chosen positioning settings to the selected buttons",
							disabled = function()
								if((type(FlexBar2.SettingCache.X) == "table" or type(FlexBar2.Buttons[FlexBar2.SettingCache.TblButtonList[1]].Settings.X) == "number") and (type(FlexBar2.SettingCache.Y) == "table" or type(FlexBar2.Buttons[FlexBar2.SettingCache.TblButtonList[1]].Settings.Y) == "number")) then
									return false
								else
									return true
								end
							end,
							order = 5,
							func = function()
								for _, v in ipairs(FlexBar2.SettingCache.TblButtonList) do
									FlexBar2.Buttons[v]:SetPosition(FlexBar2.SettingCache.X[v], FlexBar2.SettingCache.Y[v]);
								end
							end,
						},
					}
				},
			}
		},
		{
			type = "group",
			name = "Set up Group",
			disabled = true,
			desc = "Setup an already existing group",
			args =
			{
			}
		}
	}
};

FlexBar2.defaultMinimapPosition = 285;
FlexBar2.title = "FlexBar 2"
FlexBar2.clickableTooltip = false;
FlexBar2.cannotDetachTooltip = true;
FlexBar2.independentProfile = true;
FlexBar2.hasNoColor = true;
FlexBar2.hasIcon = "Interface\\Icons\\Spell_Nature_Strength";

local Tablet = AceLibrary("Tablet-2.0");
function FlexBar2:OnTooltipUpdate()
	local ClickActions = Tablet:AddCategory()
	Tablet:SetHint("");
end

function FlexBar2:ButtonParser(Buttons, validate)
	RangeStart, RangeEnd = string.match(Buttons, "^(%d+)-(%d+)$");
	if(RangeStart and RangeEnd) then
		if(validate) then
			return true;
		else
			local Range = {};
			for i = RangeStart, RangeEnd do
				table.insert(Range, i);
			end
			return Range;
		end
	end
	if(string.match(Buttons, "^%[(.+)%]$")) then
		if(validate) then
			return true;
		else
			local List =  { string.split(" ", string.sub(Buttons, 2, -2)) };
			for n, v in ipairs(List) do
				v = tonumber(v);
				if(v ~= nil) then
					List[n] = v;
				end
			end
		end
	end
	Button = string.match(Buttons, "^(%w+)$");
	if(Button) then
		if(tonumber(Button) ~= nil) then
			Button = tonumber(Button);
		end
		return { Button };
	end
	return false;
end
